/******************************************************************************
 * Beispiel:    simple
 * 
 * Beschreibung:  Dieses Beispiel zeigt die Ansteuerung der SD-Karte und 
 *                des MP3-Decoders.
 *                Dabei wird die Datei "001.mp3" von der SD-Karte des Raspberry Pi gelesen
 *                und über den MP3-Decoder ausgegeben.
 ******************************************************************************/
#include "arduPi.h"
#include "AudioShield.h"

int main (){
  setup();
  while(1){
    loop();
  }
  return (0);
}

/******************************************************************************
 * Funktionen
 ******************************************************************************/
void setup() {
  // //Einrichten der LEDs
  // pinMode(LED_BLUE, OUTPUT);
  // pinMode(LED_RED, OUTPUT);
  
  // //beide LEDs ausschalten
  // LED_BLUE_OFF;
  // LED_RED_OFF;
  
  //MP3-Decoder initialisieren
  VS1011.begin();
}

void loop()
{
  //Datei anlegen
  FILE *SoundFile;
  //Puffer für MP3-Decoder anlegen
  //MP3-Decoder erwartet Daten immer in 32 Byte Blöcken
  unsigned char buffer[32];
  //Zähler für Bytes in Puffer
  unsigned char count;
  
  //Datei öffnen und abspielen  
  SoundFile = fopen( "001.mp3", "r" );
  if( SoundFile )
  {
    printf("File start\n");
    // //Blaue LED während dem Abspielen leuchten lassen
    // LED_BLUE_ON; 
    
    //Verstärker einschalten
    VS1011.UnsetMute();
    
    //Datei bis zum Ende abspielen
    while( !feof(SoundFile) )
    {
      //Puffer mit Daten aus der Datei füllen
      count = fread(&buffer[0], sizeof(unsigned char), sizeof(buffer), SoundFile);
      //Rest des Puffers mit Nullen auffüllen
      if(count < 32)
      {
        for( unsigned char i = count; i < 32; i++)
        {
          buffer[i] = 0;
        }
      }
      //Daten aus Puffer an MP3-Decoder senden
      VS1011.Send32( &buffer[0] );
    }
    //Internen Datenpuffer vom MP3-Decoder mit Nullen füllen
    //damit sicher alles im Puffer abgespielt wird und Puffer leer ist
    //MP3-Decoder besitzt 2048 Byte großen Datenpuffer
    VS1011.Send2048Zeros();
    
    //Verstärker deaktivieren
    VS1011.SetMute();
    
    // //Blaue LED nach dem Abspielen ausschalten
    // LED_BLUE_OFF;
    
    //Datei schliessen
    fclose(SoundFile);
    printf("End of File\n");
  }
  else
  {
    //Datei nicht gefunden
    printf("File not found\n");
    // //Rote LED für 2s leuchten lassen
    // LED_RED_ON;
    // delay( 2000 );
    // LED_RED_OFF;
  }
}