/******************************************************************************
Beispiel:       SettingAlarm

Beschreibung:   Dieses Beispiel zeigt die Verwendung des Alarms innerhalb des RTCs
******************************************************************************/
#include "arduPi.h"
#include "TwoWireRTC.h"
#include "DateTime.h"

int main (){
  setup();
  while(1){
    loop();
  }
  return (0);
}

/******************************************************************************
globale Variablen
******************************************************************************/
// Datum-Uhrzeit-Struktur mit Zeit, die 5 Sekunden vor dem Alarmwert liegt
DateTime date_time = DateTime(11, 1, 1, DateTime::SATURDAY, 0, 0, 55);

// Anzeige, ob ein Alarm eingegangen ist (wird durch den Interrupt des RTCs gesetzt) 
volatile unsigned char alarmFlag = 0;

/******************************************************************************
Funktionen
******************************************************************************/

/*---------------------------------------------------------
Name:           alarm

Beschreibung:   wird ausgeführt, wenn ein externer Interrupt
                durch den RTC-Chip ausgelöst wurde
 
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
void alarm(void)
{
  alarmFlag = 1;
}

void setup() 
{
  // den Interrupt-Pin INTRA des RTC-Chips als externe Interruptquelle
  // einstellen
  pinMode(2, INPUT);
  digitalWrite(2, HIGH); 
  attachInterrupt(2, alarm, FALLING);
  
  Wire.begin();
  RTC.begin();
  
  // die Alarmuhrzeit einstellen
  RTC.setAlarmTime(ALARM_A, SATURDAY, 0, 1);
  RTC.activateAlarm(ALARM_A);
  RTC.resetAlarm(ALARM_A);
  
  // setzen der Uhrzeit (liegt 5 Sekunden vor dem Alarmwert)
  RTC.setTime(&date_time);
}

void loop() 
{
  // geht ein Alarm ein, wird dieses über die serielle Schnittstelle
  // ausgegeben und der Alarm zurückgesetzt (wieder scharfgeschaltet).
  if(alarmFlag)
  {
    alarmFlag = 0;
    
    RTC.resetAlarm(ALARM_A);
    
    printf("ALARM\n");
  }
}