/******************************************************************************
Beispiel:       SettingTime

Beschreibung:   Dieses Beispiel zeigt die Einstellung und das Auslesen der Uhrzeit
                innerhalb des RTC-Chips.
                
                Hinweis:
                Dieses Beispiel soll zeigen wie die Uhrzeit des RTC-Chips gesetzt
                bzw. ausgelesen wird. Dieses wird hier durch eine zyklische Abfrage
                des RTC-Chips (ca. 1 Sekunde) realisiert. Um dieses Beispiel jedoch
                einfach zu halten (ohne Interrupts), wurde hier mit delay gearbeitet,
                so dass es dazu kommen kann, dass zwischen zwei Ausgaben über die
                serielle Schnittstelle eine Differenz von zwei Sekunden entsteht.
                Dieses ist in der Ungenauigkeit der delay-Funktion begründet.
                Für einen genaueren Sekundentakt sei hier auf das Beispiel 
                "periodicInterrupt" verwiesen.
******************************************************************************/
#include "arduPi.h"
#include "TwoWireRTC.h"
#include "DateTime.h"

int main (){
	setup();
	while(1){
		loop();
	}
	return (0);
}

/******************************************************************************
globale Variablen
******************************************************************************/
// einzustellende Uhrzeit
DateTime date_time = DateTime(11, 1, 1, DateTime::SATURDAY, 0, 0, 0);

/******************************************************************************
Funktionen
******************************************************************************/
void setup() 
{
  Wire.begin();
  RTC.begin();

  // Uhrzeit im RTC-Chip setzen
  RTC.setTime(&date_time);
}

void loop() 
{
  // ca. jede Sekunde (beachte Hinweis oben) wird die Uhrzeit ausgelesen und über
  // die serielle Schnittstelle ausgegeben.
  delay(1000);
  
  RTC.getTime(&date_time);
  
  printf("aktuelle RTC-Zeit: %02u.%02u.%02u %02u:%02u:%02u\n",date_time.getDay(),date_time.getMonth(),date_time.getYear(),date_time.getHour(),date_time.getMinute(),date_time.getSecond());
  // printf(date_time.getDay(), DEC);
  // printf(".");
  // printf(date_time.getMonth(), DEC); 
  // printf(".");
  // printf(date_time.getYear(), DEC); 
  // printf(" ");
  // printf(date_time.getHour(), DEC);
  // printf(":");
  // printf(date_time.getMinute(), DEC);
  // printf(":");
  // printf(date_time.getSecond(), DEC);
  // printf("\n");
}

