/******************************************************************************
Beispiel:       periodicInterrupt

Beschreibung:   Dieses Beispiel zeigt den Betrieb mit periodischem Interupt,
                der jede Sekunde auslöst und dadurch die Uhrzeit um eine Sekunde
                erhöht.
******************************************************************************/
#include "arduPi.h"
#include "TwoWireRTC.h"
#include "DateTime.h"

int main (){
  setup();
  while(1){
    loop();
  }
  return (0);
}

/******************************************************************************
globale Variablen
******************************************************************************/
// Datum-Uhrzeit-Struktur zur Darstellung der laufenden Uhr
DateTime date_time = DateTime(11, 1, 1, 5, 0, 0, 0);

// Anzeige, ob eine Sekunde vergangen ist (wird durch den Interrupt des RTCs gesetzt) 
volatile unsigned char newSecondFlag = 0;

/******************************************************************************
Funktionen
******************************************************************************/

/*---------------------------------------------------------
Name:           newSecond

Beschreibung:   wird ausgeführt, wenn ein externer Interrupt
                durch den RTC-Chip ausgelöst wurde
 
Eingänge:       void

Ausgang:        void
---------------------------------------------------------*/
void newSecond(void)
{
  newSecondFlag = 1;
}

void setup() 
{
  // den Interrupt-Pin INTRA des RTC-Chips als externe Interruptquelle
  // einstellen
  pinMode(2, INPUT);
  digitalWrite(2, HIGH);
  attachInterrupt(2, newSecond, FALLING);
  
  Wire.begin();
  RTC.begin();
  
  // den RTC auf einen periodischen Interrupt von 1 Hz einstellen
  RTC.setPeriodicInterrupt(PERIODIC_INTERRUPT_PULSE_MODE_1_HZ);
}

void loop() 
{
  // nach jedem Interrupt die Sekunden um eins erhöhen und die
  // Uhrzeit über die serielle Schnittstelle ausgeben.
  if(newSecondFlag)
  {
    newSecondFlag = 0;
    date_time.addSecond();
    printf("interne Zeit: %02u.%02u.%02u %02u:%02u:%02u\n",date_time.getDay(),date_time.getMonth(),date_time.getYear(),date_time.getHour(),date_time.getMinute(),date_time.getSecond());
  }
}