#ifndef CP240X_REG_H
#define CP240X_REG_H

// SmaRTClock Registers
#define CP240X_RTCKEY                       0x0A  //  RTC0 Indirect Address
#define CP240X_RTCADR                       0x0B  //  RTC0 Indirect Data
#define CP240X_RTCDAT                       0x0C  //  RTC0 Lock and Key

// Interrupt Mask and Clocking Registers
#define CP240X_INT0EN                       0x30  //  Interrupt Enable Register 0
#define CP240X_INT1EN                       0x31  //  Interrupt Enable Register 1
#define CP240X_CLKSL                        0x32  //  Clock Select
#define CP240X_IOSCCN                       0x33  //  Internal Oscillator Control
#define CP240X_REVID                        0x34  //  Revision Identifier

// Interrupt Status Registers
#define CP240X_INT0RD                       0x40  //  Interrupt Status Register 0 (read-only)
#define CP240X_INT1RD                       0x41  //  Interrupt Status Register 1 (read-only)
#define CP240X_ULPST                        0x42  //  Ultra Low Power Status
#define CP240X_INT0                         0x43  //  Interrupt Status Register 0 (self-clearing)
#define CP240X_INT1                         0x44  //  Interrupt Status Register 1 (self-clearing)

// Timer 0 and Timer 1 Registers
#define CP240X_TMR0RLL                      0x50  //  Timer 0 Reload Register Low Byte
#define CP240X_TMR0RLH                      0x51  //  Timer 0 Reload Register High Byte
#define CP240X_TMR0L                        0x52  //  Timer 0 Low Byte
#define CP240X_TMR0H                        0x53  //  Timer 0 High Byte
#define CP240X_TMR0CN                       0x54  //  Timer 0 Control
#define CP240X_TMR1RLL                      0x55  //  Timer 1 Reload Register Low Byte
#define CP240X_TMR1RLH                      0x56  //  Timer 1 Reload Register High Byte
#define CP240X_TMR1L                        0x57  //  Timer 1 Low Byte
#define CP240X_TMR1H                        0x58  //  Timer 1 High Byte
#define CP240X_TMR1CN                       0x59  //  Timer 1 Control

// SMBus Registers
#define CP240X_SMBCF                        0x68  //  SMBus Configuration

// ULP/LCD0 Data Registers
#define CP240X_LCD0BLINK                    0x80  //  LCD0 Segment Blink
#define CP240X_ULPMEM00                     0x81  //  ULP Memory Byte 0
#define CP240X_ULPMEM01                     0x82  //  ULP Memory Byte 1
#define CP240X_ULPMEM02                     0x83  //  ULP Memory Byte 2
#define CP240X_ULPMEM03                     0x84  //  ULP Memory Byte 3
#define CP240X_ULPMEM04                     0x85  //  ULP Memory Byte 4
#define CP240X_ULPMEM05                     0x86  //  ULP Memory Byte 5
#define CP240X_ULPMEM06                     0x87  //  ULP Memory Byte 6
#define CP240X_ULPMEM07                     0x88  //  ULP Memory Byte 7
#define CP240X_ULPMEM08                     0x89  //  ULP Memory Byte 8
#define CP240X_ULPMEM09                     0x8A  //  ULP Memory Byte 9
#define CP240X_ULPMEM10                     0x8B  //  ULP Memory Byte 10
#define CP240X_ULPMEM11                     0x8C  //  ULP Memory Byte 11
#define CP240X_ULPMEM12                     0x8D  //  ULP Memory Byte 12
#define CP240X_ULPMEM13                     0x8E  //  ULP Memory Byte 13
#define CP240X_ULPMEM14                     0x8F  //  ULP Memory Byte 14
#define CP240X_ULPMEM15                     0x90  //  ULP Memory Byte 15

// LCD Control Registers
#define CP240X_LCD0CN                       0x95  //  LCD0 Control
#define CP240X_CONTRAST                     0x96  //  LCD0 Contrast Adjustment
#define CP240X_LCD0CF                       0x97  //  LCD0 Configuration
#define CP240X_LCD0DIVL                     0x98  //  LCD0 Clock Divider High Byte
#define CP240X_LCD0DIVH                     0x99  //  LCD0 Clock Divider Low Byte
#define CP240X_LCD0TOGR                     0x9A  //  LCD0 Toggle Rate
#define CP240X_LCD0PWR                      0x9B  //  LCD0 Power Mode

// Ultra Low Power Control Registers
#define CP240X_MSCN                         0xA0  //  Master Control
#define CP240X_MSCF                         0xA1  //  Master Configuration
#define CP240X_ULPCN                        0xA2  //  Ultra Low Power Control

// Port I/O Configuration Registers
#define CP240X_P0OUT                        0xB0  //  Port 0 Output Data Latch
#define CP240X_P1OUT                        0xB1  //  Port 1 Output Data Latch
#define CP240X_P2OUT                        0xB2  //  Port 2 Output Data Latch
#define CP240X_P3OUT                        0xB3  //  Port 3 Output Data Latch
#define CP240X_P4OUT                        0xB4  //  Port 4 Output Data Latch
#define CP240X_P0MDI                        0xB5  //  Port 0 Input Mode
#define CP240X_P1MDI                        0xB6  //  Port 1 Input Mode
#define CP240X_P2MDI                        0xB7  //  Port 2 Input Mode
#define CP240X_P3MDI                        0xB8  //  Port 3 Input Mode
#define CP240X_P4MDI                        0xB9  //  Port 4 Input Mode
#define CP240X_P0MDO                        0xBA  //  Port 0 Output Mode
#define CP240X_P1MDO                        0xBB  //  Port 1 Output Mode
#define CP240X_P2MDO                        0xBC  //  Port 2 Output Mode
#define CP240X_P3MDO                        0xBD  //  Port 3 Output Mode
#define CP240X_P4MDO                        0xBE  //  Port 4 Output Mode
#define CP240X_P0DRIVE                      0xBF  //  Port 0 Drive Strength
#define CP240X_P1DRIVE                      0xC0  //  Port 1 Drive Strength
#define CP240X_P2DRIVE                      0xC1  //  Port 2 Drive Strength
#define CP240X_P3DRIVE                      0xC2  //  Port 3 Drive Strength
#define CP240X_P4DRIVE                      0xC3  //  Port 4 Drive Strength
#define CP240X_P0MATCH                      0xC4  //  Port 0 Match
#define CP240X_P1MATCH                      0xC5  //  Port 1 Match
#define CP240X_P2MATCH                      0xC6  //  Port 2 Match
#define CP240X_P3MATCH                      0xC7  //  Port 3 Match
#define CP240X_P4MATCH                      0xC8  //  Port 4 Match
#define CP240X_P0MSK                        0xC9  //  Port 0 Mask
#define CP240X_P1MSK                        0xCA  //  Port 1 Mask
#define CP240X_P2MSK                        0xCB  //  Port 2 Mask
#define CP240X_P3MSK                        0xCC  //  Port 3 Mask
#define CP240X_P4MSK                        0xCD  //  Port 4 Mask

// Port I/O Input and Status Registers
#define CP240X_PMATCHST                     0xD0  //  Port Match Status
#define CP240X_P0IN                         0xD1  //  Port 0 Input
#define CP240X_P1IN                         0xD2  //  Port 1 Input
#define CP240X_P2IN                         0xD3  //  Port 2 Input
#define CP240X_P3IN                         0xD4  //  Port 3 Input
#define CP240X_P4IN                         0xD5  //  Port 4 Input

//-----------------------------------------------------------------------------
// SmaRTClock Register Definitions
//-----------------------------------------------------------------------------
#define CAPTURE0  0x00                 // RTC address of CAPTURE0 register
#define CAPTURE1  0x01                 // RTC address of CAPTURE1 register
#define CAPTURE2  0x02                 // RTC address of CAPTURE2 register
#define CAPTURE3  0x03                 // RTC address of CAPTURE3 register
#define RTC0CN    0x04                 // RTC address of RTC0CN register                
#define RTC0XCN   0x05                 // RTC address of RTC0XCN register 
#define RTC0XCF   0x06                 // RTC address of RTC0XCF register
#define ALARM0    0x08                 // RTC address of ALARM0 register
#define ALARM1    0x09                 // RTC address of ALARM1 register
#define ALARM2    0x0A                 // RTC address of ALARM2 register
#define ALARM3    0x0B                 // RTC address of ALARM3 register

// SmaRTClock Bit Definitions
#define RTC0CAP   0x01
#define RTC0SET   0x02
#define ALRM      0x04
#define RTC0AEN   0x08
#define RTC0TR    0x10
#define OSCFAIL   0x20
#define MCLKEN    0x40
#define RTC0EN    0x80

//-----------------------------------------------------------------------------
// Register Accessing Commands
//-----------------------------------------------------------------------------
#define REGPOLL     0x01
#define REGREAD     0x02
#define REGSET      0x03
#define REGWRITE    0x04

#endif /* CP240X_REG_H */
